%---------------------------------------------------------------------------%

:- module fruitbowl.
:- interface.

:- import_module io.

%---------------------------------------------------------------------------%

:- type database.

:- type fruit == string.

:- type is_citrus
    --->   citrus
    ;      not_citrus.

:- func init_database = database.

    % Add a fruit to the database, overwriting an existing entry
    % for that fruit if present.
    %
:- pred add(fruit::in, is_citrus::in, database::in, database::out) is det.

    % Remove a fruit from the database; does nothing if there is no
    % existing entry for the given fruit in the database.
    %
:- pred remove(fruit::in, database::in, database::out) is det.

    % True if the database contains an entry for the given fruit.
    %
:- pred is_fruit(database::in, fruit::in) is semidet.

    % True if the database contains an entry for the given fruit
    % *and* that fruit is a citrus fruit.
    %
:- pred is_citrus(database::in, fruit::in) is semidet.

    % Print the database to the current Mercury text output stream.
    %
:- pred print_database(database::in, io::di, io::uo) is det.

%---------------------------------------------------------------------------%
%---------------------------------------------------------------------------%

:- implementation.

:- import_module list.
:- import_module map.
:- import_module string.

%---------------------------------------------------------------------------%

:- type database == map(fruit, is_citrus).

:- pragma foreign_export_enum("C#", is_citrus/0, [uppercase]).

%---------------------------------------------------------------------------%

:- pragma foreign_export("C#", init_database = out, "InitDatabase").

init_database = map.init.

:- pragma foreign_export("C#", add(in, in, in, out), "Add").

add(Fruit, IsCitrus, !DB) :-
    map.set(Fruit, IsCitrus, !DB).

:- pragma foreign_export("C#", remove(in, in, out), "Remove").

remove(Fruit, !DB) :-
    map.delete(Fruit, !DB).

:- pragma foreign_export("C#", is_citrus(in, in), "IsCitrus").

is_citrus(DB, Fruit) :-
    map.search(DB, Fruit, citrus).

:- pragma foreign_export("C#", is_fruit(in, in), "IsFruit").

is_fruit(DB, Fruit) :-
    map.contains(DB, Fruit).

:- pragma foreign_export("C#", print_database(in, di, uo), "PrintDatabase").

print_database(DB, !IO) :-
    io.print_line("------ DB -------", !IO),
    ( if map.is_empty(DB) then
        io.print_line("Database is empty.", !IO)
    else
        map.foldl(print_entry, DB, !IO)
    ),
    io.print_line("-----------------", !IO).

:- pred print_entry(fruit::in, is_citrus::in, io::di, io::uo) is det.

print_entry(Fruit, IsCitrus, !IO) :-
    io.format("%s - %s\n", [s(Fruit), s(string(IsCitrus))], !IO).

%---------------------------------------------------------------------------%
:- end_module fruitbowl.
%---------------------------------------------------------------------------%
